

package com.office.allreader.allofficefilereader.fc.hpsf;

import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;

import com.office.allreader.allofficefilereader.fc.POIDocument;
import com.office.allreader.allofficefilereader.fc.POITextExtractor;
import com.office.allreader.allofficefilereader.fc.poifs.filesystem.NPOIFSFileSystem;
import com.office.allreader.allofficefilereader.fc.poifs.filesystem.POIFSFileSystem;
import com.office.allreader.allofficefilereader.fc.util.LittleEndian;



/**
 * Extracts all of the HPSF properties, both
 *  build in and custom, returning them in
 *  textual form.
 */
public class HPSFPropertiesExtractor extends POITextExtractor {
	public HPSFPropertiesExtractor(POITextExtractor mainExtractor) {
		super(mainExtractor);
	}
	public HPSFPropertiesExtractor(POIDocument doc) {
		super(doc);
	}
	public HPSFPropertiesExtractor(POIFSFileSystem fs) {
		super(new PropertiesOnlyDocument(fs));
	}
   public HPSFPropertiesExtractor(NPOIFSFileSystem fs) {
      super(new PropertiesOnlyDocument(fs));
   }

	public String getDocumentSummaryInformationText() {
		DocumentSummaryInformation dsi = document.getDocumentSummaryInformation();
		StringBuffer text = new StringBuffer();

		// Normal properties
		text.append( getPropertiesText(dsi) );

		// Now custom ones
		CustomProperties cps = dsi == null ? null : dsi.getCustomProperties();
		if(cps != null) {
			Iterator<String> keys = cps.nameSet().iterator();
			while(keys.hasNext()) {
				String key = keys.next();
				String val = getPropertyValueText( cps.get(key) );
				text.append(key + " = " + val + "\n");
			}
		}

		// All done
		return text.toString();
	}
	public String getSummaryInformationText() {
		SummaryInformation si = document.getSummaryInformation();

		// Just normal properties
		return getPropertiesText(si);
	}

	private static String getPropertiesText(SpecialPropertySet ps) {
		if(ps == null) {
			// Not defined, oh well
			return "";
		}

		StringBuffer text = new StringBuffer();

		PropertyIDMap idMap = ps.getPropertySetIDMap();
		Property[] props = ps.getProperties();
		for(int i=0; i<props.length; i++) {
			String type = Long.toString( props[i].getID() );
			Object typeObj = idMap.get(props[i].getID());
			if(typeObj != null) {
				type = typeObj.toString();
			}

			String val = getPropertyValueText( props[i].getValue() );
			text.append(type + " = " + val + "\n");
		}

		return text.toString();
	}
	private static String getPropertyValueText(Object val) {
		if(val == null) {
			return "(not set)";
		}
		if(val instanceof byte[]) {
			byte[] b = (byte[])val;
			if(b.length == 0) {
				return "";
			}
			if(b.length == 1) {
				return Byte.toString(b[0]);
			}
			if(b.length == 2) {
				return Integer.toString( LittleEndian.getUShort(b) );
			}
			if(b.length == 4) {
				return Long.toString( LittleEndian.getUInt(b) );
			}
			// Maybe it's a string? who knows!
			return new String(b);
		}
		return val.toString();
	}

	/**
	 * @return the text of all the properties defined in
	 *  the document.
	 */
	public String getText() {
		return getSummaryInformationText() + getDocumentSummaryInformationText();
	}

	/**
	 * Prevent recursion!
	 */
	public POITextExtractor getMetadataTextExtractor() {
		throw new IllegalStateException("You already have the Metadata Text Extractor, not recursing!");
	}

	/**
	 * So we can get at the properties of any
	 *  random OLE2 document.
	 */
	private static final class PropertiesOnlyDocument extends POIDocument {
      public PropertiesOnlyDocument(NPOIFSFileSystem fs) {
         super(fs.getRoot());
      }
		public PropertiesOnlyDocument(POIFSFileSystem fs) {
			super(fs);
		}

		public void write(OutputStream out) {
			throw new IllegalStateException("Unable to write, only for properties!");
		}
	}
	
	public static void main(String[] args) throws IOException {
	   for(String file : args) {
	      HPSFPropertiesExtractor ext = new HPSFPropertiesExtractor(
	            new NPOIFSFileSystem(new File(file))
	      );
	      System.out.println(ext.getText());
	   }
	}
}
