

package com.office.allreader.allofficefilereader.fc.hslf.blip;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.zip.DeflaterOutputStream;
import java.util.zip.InflaterInputStream;

import com.office.allreader.allofficefilereader.fc.hslf.exceptions.HSLFException;
import com.office.allreader.allofficefilereader.fc.hslf.usermodel.PictureData;
import com.office.allreader.allofficefilereader.fc.util.LittleEndian;
import com.office.allreader.allofficefilereader.java.awt.Dimension;
import com.office.allreader.allofficefilereader.java.awt.Rectangle;


/**
 * Represents a metafile picture which can be one of the following types: EMF, WMF, or PICT.
 * A metafile is stored compressed using the ZIP deflate/inflate algorithm.
 *
 * @author Yegor Kozlov
 */
public abstract class Metafile extends PictureData
{

    /**
     *  A structure which represents a 34-byte header preceeding the compressed metafile data
     *
     * @author Yegor Kozlov
     */
    public static class Header
    {

        /**
         * size of the original file
         */
        public int wmfsize;

        /**
         * Boundary of the metafile drawing commands
         */
        public Rectangle bounds;

        /**
         *  Size of the metafile in EMUs
         */
        public Dimension size;

        /**
         * size of the compressed metafile data
         */
        public int zipsize;

        /**
         * Reserved. Always 0.
         */
        public int compression;

        /**
         * Reserved. Always 254.
         */
        public int filter = 254;

        public void read(byte[] data, int offset)
        {
            int pos = offset;
            wmfsize = LittleEndian.getInt(data, pos);
            pos += LittleEndian.INT_SIZE;

            int left = LittleEndian.getInt(data, pos);
            pos += LittleEndian.INT_SIZE;
            int top = LittleEndian.getInt(data, pos);
            pos += LittleEndian.INT_SIZE;
            int right = LittleEndian.getInt(data, pos);
            pos += LittleEndian.INT_SIZE;
            int bottom = LittleEndian.getInt(data, pos);
            pos += LittleEndian.INT_SIZE;

            bounds = new Rectangle(left, top, right - left, bottom - top);
            int width = LittleEndian.getInt(data, pos);
            pos += LittleEndian.INT_SIZE;
            int height = LittleEndian.getInt(data, pos);
            pos += LittleEndian.INT_SIZE;

            size = new Dimension(width, height);

            zipsize = LittleEndian.getInt(data, pos);
            pos += LittleEndian.INT_SIZE;

            compression = LittleEndian.getUnsignedByte(data, pos);
            pos++;
            filter = LittleEndian.getUnsignedByte(data, pos);
            pos++;
        }

        public void write(OutputStream out) throws IOException
        {
            byte[] header = new byte[34];
            int pos = 0;
            LittleEndian.putInt(header, pos, wmfsize);
            pos += LittleEndian.INT_SIZE; //hmf

            LittleEndian.putInt(header, pos, bounds.x);
            pos += LittleEndian.INT_SIZE; //left
            LittleEndian.putInt(header, pos, bounds.y);
            pos += LittleEndian.INT_SIZE; //top
            LittleEndian.putInt(header, pos, bounds.x + bounds.width);
            pos += LittleEndian.INT_SIZE; //right
            LittleEndian.putInt(header, pos, bounds.y + bounds.height);
            pos += LittleEndian.INT_SIZE; //bottom
            LittleEndian.putInt(header, pos, size.width);
            pos += LittleEndian.INT_SIZE; //inch
            LittleEndian.putInt(header, pos, size.height);
            pos += LittleEndian.INT_SIZE; //inch
            LittleEndian.putInt(header, pos, zipsize);
            pos += LittleEndian.INT_SIZE; //inch

            header[pos] = 0;
            pos++;
            header[pos] = (byte)filter;
            pos++;

            out.write(header);
        }

        public int getSize()
        {
            return 34;
        }
    }

    protected byte[] compress(byte[] bytes, int offset, int length) throws IOException
    {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        DeflaterOutputStream deflater = new DeflaterOutputStream(out);
        deflater.write(bytes, offset, length);
        deflater.close();
        return out.toByteArray();
    }
    
    public void writeByte_WMFAndEMF(FileOutputStream out)
    {
    	try
        {
            byte[] rawdata = getRawData();

            InputStream is = new ByteArrayInputStream(rawdata);
            is.skip(8);
            
            Header header = new Header();
            header.read(rawdata, CHECKSUM_SIZE);
            is.skip(header.getSize() + CHECKSUM_SIZE);

            InflaterInputStream inflater = new InflaterInputStream(is);
            byte[] chunk = new byte[4096];
            int count;
            while ((count = inflater.read(chunk)) >= 0)
            {
                out.write(chunk, 0, count);
            }
            inflater.close();
        }
        catch(IOException e)
        {
            throw new HSLFException(e);
        }
    }
}
