

package com.office.allreader.allofficefilereader.fc.hssf.formula.function;


import com.office.allreader.allofficefilereader.fc.hssf.formula.eval.AreaEval;
import com.office.allreader.allofficefilereader.fc.hssf.formula.eval.ErrorEval;
import com.office.allreader.allofficefilereader.fc.hssf.formula.eval.EvaluationException;
import com.office.allreader.allofficefilereader.fc.hssf.formula.eval.NumberEval;
import com.office.allreader.allofficefilereader.fc.hssf.formula.eval.RefEval;
import com.office.allreader.allofficefilereader.fc.hssf.formula.eval.ValueEval;
import com.office.allreader.allofficefilereader.fc.hssf.formula.function.CountUtils.I_MatchPredicate;


/**
 * Implementation for the Excel function SUMIF<p>
 *
 * Syntax : <br/>
 *  SUMIF ( <b>range</b>, <b>criteria</b>, sum_range ) <br/>
 *    <table border="0" cellpadding="1" cellspacing="0" summary="Parameter descriptions">
 *      <tr><th>range</th><td>The range over which criteria is applied.  Also used for addend values when the third parameter is not present</td></tr>
 *      <tr><th>criteria</th><td>The value or expression used to filter rows from <b>range</b></td></tr>
 *      <tr><th>sum_range</th><td>Locates the top-left corner of the corresponding range of addends - values to be added (after being selected by the criteria)</td></tr>
 *    </table><br/>
 * </p>
 * @author Josh Micich
 */
public final class Sumif extends Var2or3ArgFunction {

	public ValueEval evaluate(int srcRowIndex, int srcColumnIndex, ValueEval arg0, ValueEval arg1) {

		AreaEval aeRange;
		try {
			aeRange = convertRangeArg(arg0);
		} catch (EvaluationException e) {
			return e.getErrorEval();
		}
		return eval(srcRowIndex, srcColumnIndex, arg1, aeRange, aeRange);
	}

	public ValueEval evaluate(int srcRowIndex, int srcColumnIndex, ValueEval arg0, ValueEval arg1,
			ValueEval arg2) {

		AreaEval aeRange;
		AreaEval aeSum;
		try {
			aeRange = convertRangeArg(arg0);
			aeSum = createSumRange(arg2, aeRange);
		} catch (EvaluationException e) {
			return e.getErrorEval();
		}
		return eval(srcRowIndex, srcColumnIndex, arg1, aeRange, aeSum);
	}

	private static ValueEval eval(int srcRowIndex, int srcColumnIndex, ValueEval arg1, AreaEval aeRange,
			AreaEval aeSum) {
		// TODO - junit to prove last arg must be srcColumnIndex and not srcRowIndex
		I_MatchPredicate mp = Countif.createCriteriaPredicate(arg1, srcRowIndex, srcColumnIndex);
		double result = sumMatchingCells(aeRange, mp, aeSum);
		return new NumberEval(result);
	}

	private static double sumMatchingCells(AreaEval aeRange, I_MatchPredicate mp, AreaEval aeSum) {
		int height=aeRange.getHeight();
		int width= aeRange.getWidth();

		double result = 0.0;
		for (int r=0; r<height; r++) {
			for (int c=0; c<width; c++) {
				result += accumulate(aeRange, mp, aeSum, r, c);
			}
		}
		return result;
	}

	private static double accumulate(AreaEval aeRange, I_MatchPredicate mp, AreaEval aeSum, int relRowIndex,
			int relColIndex) {

		if (!mp.matches(aeRange.getRelativeValue(relRowIndex, relColIndex))) {
			return 0.0;
		}
		ValueEval addend = aeSum.getRelativeValue(relRowIndex, relColIndex);
		if (addend instanceof NumberEval) {
			return ((NumberEval)addend).getNumberValue();
		}
		// everything else (including string and boolean values) counts as zero
		return 0.0;
	}

	/**
	 * @return a range of the same dimensions as aeRange using eval to define the top left corner.
	 * @throws EvaluationException if eval is not a reference
	 */
	private static AreaEval createSumRange(ValueEval eval, AreaEval aeRange) throws EvaluationException {
		if (eval instanceof AreaEval) {
			return ((AreaEval) eval).offset(0, aeRange.getHeight()-1, 0, aeRange.getWidth()-1);
		}
		if (eval instanceof RefEval) {
			return ((RefEval)eval).offset(0, aeRange.getHeight()-1, 0, aeRange.getWidth()-1);
		}
		throw new EvaluationException(ErrorEval.VALUE_INVALID);
	}

	private static AreaEval convertRangeArg(ValueEval eval) throws EvaluationException {
		if (eval instanceof AreaEval) {
			return (AreaEval) eval;
		}
		if (eval instanceof RefEval) {
			return ((RefEval)eval).offset(0, 0, 0, 0);
		}
		throw new EvaluationException(ErrorEval.VALUE_INVALID);
	}

}
